import pathlib

import requests

from pukpuk.core.mods import HttpModule
from pukpuk.core import logging


class Module(HttpModule):

    name = 'Response'

    def before(self):
        if self.main.proxy:
            socks_str = f'socks5h://{self.main.proxy[0]}:{self.main.proxy[1]}'
            self.cache['proxies'] = {
                'http': socks_str,
                'https': socks_str,
            }

    def execute(self, url):
        mod_dir = pathlib.Path(self.main.args.output_directory, self.name.lower())
        mod_dir.mkdir(parents=True, exist_ok=True)
        base_filename = pathlib.Path(mod_dir, self.get_base_filename(url))
        get_args = {
            'verify': False,
            'timeout': self.main.args.socket_timeout,
            'headers': self.main.headers,
        }
        if self.main.proxy:
            get_args['proxies'] = self.cache['proxies']
        try:
            response = requests.get(url, **get_args)
        except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout):
            logging.logger.debug(f'Could not retrieve {url}')
        except requests.exceptions.InvalidURL:
            logging.logger.info(f'Invalid URL: {url}')
        else:
            with open(f'{base_filename}.txt', 'wb') as fil:
                fil.write('\n'.join([header + ': ' + value for header, value in response.headers.items()]).encode('utf8'))
                fil.write('\n\n'.encode('utf8'))
                fil.write(response.content)
            logging.logger.info(f'{self.name} finished {url}')



from pukpuk.core.logging import logger


class BaseModule:

    def __init__(self, *args, **kwargs):
        self.main = None
        self.cache = dict()

    def init(self, main):
        logger.debug(f'Initializing {self.__module__}')
        self.main = main
        self.before()

    def before(self):
        pass

    def execute(self):
        raise NotImplementedError


class HttpModule(BaseModule):

    def get_base_filename(self, url):
        return url.replace('://', '-').replace(':', '-')[:-1]



import logging
import pathlib
import sys


logger = logging.getLogger('pukpuk')
formatter = {
    logging.DEBUG: logging.Formatter('%(name)s %(levelname)s [%(asctime)s] %(message)s'),
    logging.INFO: logging.Formatter('[%(asctime)s] %(message)s')
}
handler = logging.StreamHandler(sys.stdout)
logger.addHandler(handler)


def init(loglevel, directory):
    handler.setLevel(loglevel)
    logger.setLevel(loglevel)
    handler.setFormatter(formatter[loglevel])
    logger.addHandler(logging.FileHandler(filename=pathlib.Path(directory, 'pukpuk.log')))



import os
import pathlib
import subprocess

from PIL import Image

from pukpuk.core.mods import HttpModule
from pukpuk.core import logging


class Module(HttpModule):

    name = 'Grabber'

    def before(self):
        if self.main.proxy:
            self.cache['proxy_server'] = 'socks5://{}:{}'.format(*self.main.proxy)

    def execute(self, url):
        user_agent = self.main.args.user_agent
        browser = self.main.args.browser
        mod_dir = pathlib.Path(self.main.args.output_directory, self.name.lower())
        mod_dir.mkdir(parents=True, exist_ok=True)
        base_filename = pathlib.Path(mod_dir, self.get_base_filename(url))
        image_filename = str(base_filename) + '.png'
        exec_args = [
            browser,
            '--headless',
            '--disable-gpu',
            '--window-size=1280,1696',
            '--v0',
            '--ignore-certificate-errors',
            f'--screenshot={image_filename}',
            f'--user-agent="{user_agent}"',
            url,
        ]
        if self.main.proxy:
            exec_args.insert(len(exec_args) - 1, '--proxy-server="{}"'.format(self.cache['proxy_server']))
            exec_args.insert(len(exec_args) - 1, '--host-resolver-rules="MAP * ~NOTFOUND , EXCLUDE {}"'.format(self.main.proxy[0]))
        try:
            subprocess.check_output(
                exec_args,
                stderr=subprocess.STDOUT,
                timeout=self.main.args.process_timeout
            )
        except FileNotFoundError:
            logging.logger.error(f'Error occured when grabbing the screen. Is `{browser}` installed?')
            exit(1)
        except subprocess.TimeoutExpired:
            logging.logger.debug(f'Screen grabbing timed out for {url} (try adjusting --process-timeout)')
        else:
            with Image.open(image_filename) as img:
                extrema = img.convert('L').getextrema()
            if extrema[0] == extrema[1]:
                os.remove(image_filename)
                logging.logger.debug(f'Blank screen for {url} returned, deleting image')
            logging.logger.info(f'{self.name} finished {url}')
